package de.uni_passau.fim.evaluator;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import de.uni_passau.fim.strategies.Configuration;
import de.uni_passau.fim.strategies.Guess;
import de.uni_passau.fim.strategies.IStrategy;
import de.uni_passau.fim.strategies.Logger;



public class Player {

	IStrategy strategy = null;
	
	int steps = 0;
	
	boolean verbose = true;
	
	/**
	 * Play a single game. Assumes that <code>m</code> already has been
	 * initialized. <code>s</code> will be resetted.
	 */
	public void play(Mastermind m, IStrategy s) {
		// TODO: Refactor thrown Exception to something more specific.
		strategy = s;
		steps = 0;
		s.reset();
		Date startOfGame = Calendar.getInstance().getTime();
		while (!m.solved() && steps < m.getRows()) {
			List<Configuration> cs = m.getCurrentState();
			Guess g = strategy.getGuess(cs);
			if (g == null) {
				Date endOfGame = Calendar.getInstance().getTime();
				float durationOfGame = endOfGame.getTime() - startOfGame.getTime();
			}
			m.guess(g);
			steps++;
		}
		Date endOfGame = Calendar.getInstance().getTime();
		float durationOfGame = endOfGame.getTime() - startOfGame.getTime();
	}
	
	/**
	 * For each element of <code>initialGuesses</code> as hidden codes, play
	 * Mastermind with the given strategy <code>s</code>.
	 * Afterwards, a short summary is printed to standard out.
	 */
	public void play(Mastermind m, IStrategy s, List<Guess> initialGuesses) {
		// Sum up the number of steps each game takes to build the average
		// afterwards.
		int cumulativeSteps = 0;
		
		// Counts the number of games where the strategy returned null.
		int faultyGames = 0;
		
		// Counts the number of games where the maximal number of guesses
		// allowed has been reached.
		int numOfNotSolved = 0;
		
		// Memorize how much steps were needed in the worst case.
		int maximumNumberOfSteps = 0;
		
		// Memorize the maximum time that were needed.
		float maximumTime = Float.MIN_VALUE;
		
		// stepCounter[i] will contain the number of games that needed
		// i steps to find a solution.
		int[] stepCounter = new int[m.getRows() + 1];
		
		
		for (int i=0; i<stepCounter.length; i++) {
			stepCounter[i] = 0;
		}
		Date startOfComputation = Calendar.getInstance().getTime();
		List<Guess> guesses = initialGuesses;
		
		int numberOfGames = guesses.size();
		int counter = 1;
		float cumulativeTime = 0;
		for (Guess guess : guesses) {
			Logger.info("Game " + counter + "/" + numberOfGames);
			Logger.info("Tested pattern: " + guess);
			
			// Initialize Mastermind with current secret code.
			m.init(guess);

			// Time the game.
			Date startOfGame = Calendar.getInstance().getTime();
			try {
				this.play(m, s);
			} catch (Exception e) {
				e.printStackTrace();
				faultyGames++;
			}
			
			Date endOfGame = Calendar.getInstance().getTime();
			float durationOfGame = endOfGame.getTime() - startOfGame.getTime();
			cumulativeTime += durationOfGame;
			
			int steps = this.getSteps();
			if (steps > maximumNumberOfSteps)
				maximumNumberOfSteps = steps;
			if (durationOfGame > maximumTime)
				maximumTime = durationOfGame;
			
			stepCounter[steps]++;
			Logger.info("Steps: " + steps);
			Logger.info("Time: " + durationOfGame/1000 + " secs.");
			Logger.info("Solved in " + m.getRows() + " steps: "
					+ (m.solved() ? "yes" : "no"));
			if (!m.solved())
				numOfNotSolved ++;
			cumulativeSteps += this.getSteps();
			counter++;
		}
		Date endOfComputation = Calendar.getInstance().getTime();
		float duration = endOfComputation.getTime() - startOfComputation.getTime();
		
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("++++++++++++   Game Info   ++++++++++++++++++++");
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("Colors: " + m.getColors() + ". Places: " 
				+ m.getPlaces() + ". Rows: " + m.getRows() +  ".");
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("Strategy info:");
		System.out.println(s.info());
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++");
		System.out.println("Number of games: " + (counter-1));
		System.out.println("Number of unsolved games: " + numOfNotSolved);
		System.out.println("Maximum # of steps: " + maximumNumberOfSteps);
		System.out.println("Average # of steps: " + ((double)cumulativeSteps/counter));
		System.out.println("Maximum time of strategy: " + (maximumTime/1000));
		System.out.println("Average time of strategy: " + (cumulativeTime/(1000*counter)));
		for (int i=0; i<stepCounter.length; i++) {
			if (stepCounter[i] != 0)
				System.out.println(i + " steps: " + stepCounter[i]);
		}
		System.out.println("Total time used: " + (duration/1000) + " seconds.");
		System.out.println("+++++++++++++++++++++++++++++++++++++++++++++++");
	}
	
	public int getSteps() {
		return steps;
	}
	
	
		
		
		/* public int numberOfGames;
		public int unsolvedGames;
		public int maxNrOfSteps;
		public double avrgNumOfSteps;
		public double maxTime;
		public double avrgTime;
		public double totalTime;
		public int[] numOfGamesForStepNum; */
	
	
	
}
