package de.uni_passau.fim.strategies;


/**
 * General representation of a guess, i.e., a combination of c colors
 * in p places. 
 * 
 */
public class Guess implements Cloneable{
	
	/** Contains colors between 0 and <code>numberOfColors-1</code> (inclusively)
	 *  and must has as length the length of the code as used by the current 
	 *  Mastermind game. */
	public int[] colors;
	
	/**colorCOunter[i] denotes how often color i occurs in colors[]. */
	private int[] colorCounter = null;
	
	/** Total number of colors which are used in the Mastermind game. */
	public int numberOfColors;
	
	private volatile int hashCode = 0;
	
	/** Creates a new guess containing with colors equal to <code>c</code>
	 * where the colors are between 0 and <code>numberOfColors-1</code>, inclusively.
	 * Note that <code>c</code> is cloned. */
	public Guess(int[] c, int numberOfColors) {
		this.colors = c.clone();
		this.numberOfColors = numberOfColors;
	}
	
	/** Returns a clone of this guess. */
	public Guess clone() {
		try {
			Guess newGuess = (Guess) super.clone();
			newGuess.colors = colors.clone();
			newGuess.colorCounter = null;
			return newGuess;
		} catch (CloneNotSupportedException e) {
			throw new InternalError(e.toString());
		}
	}
	
	/** Returns a string of the form <code>[1,2,0,0]</code>. */
	public String toString() {
		StringBuffer sb = new StringBuffer();
		sb.append("[");
		for (int i=0; i<colors.length; i++) {
			sb.append(colors[i]);
			if (i < colors.length-1)
				sb.append(", ");
		}
		sb.append("]");
		return sb.toString();
	}
	
	/** Returns true only if <code>o</code> is an guess with the same
	 * color array. */
	public boolean equals(Object o) {
		if (this == o) 
			return true;
		if (!(o instanceof Guess)) 
			return false;
		Guess g = (Guess)o;
		
		if (this.colors == g.colors) 
			return true;
		if (this.colors == null || g.colors == null) 
			return false;
		
		// Now we can assueme both .colors arrays to be non-null
		if (this.colors.length != g.colors.length) 
			return false;
		
		boolean eqls = true;
		for (int i=0; i<this.colors.length; i++)
			eqls &= this.colors[i] == g.colors[i];
		return eqls;
	}
	
	/** Returns the hash code of this guess. */
	public int hashCode() {
		if (this.colors == null)
			return 0;
		if (this.hashCode == 0) {
			int result = 17;
			for (int i=0; i<this.colors.length; i++)
				result = result*37 + this.colors[i];
			this.hashCode = result;
		}
		return this.hashCode;
	}
	
	/**
	 * If there are c colors, an array a of length c is returned, s.t.
	 * a[i] is the number of occurrences of color i in this guess.
	 *  
	 * @return array with the color count of the current guess.
	 */
	public int[] getColorCounter() {
		if (this.colorCounter == null) {
			int[] tmpCcnter = new int[numberOfColors];
			for (int i=0; i<tmpCcnter.length; i++)
				tmpCcnter[i] = 0;
			for (int i=0; i<colors.length; i++) {
				try {
					(tmpCcnter[colors[i]])++;
				} catch (ArrayIndexOutOfBoundsException e) {
					System.out.println("i = " + i);
					System.out.println("numberOfColors = " + numberOfColors);
					System.out.println("colors array:");
					for (int j=0; j<colors.length; j++) {
						System.out.println(colors[j]);
					}
					System.out.println("tmpCcntr array:");
					for (int j=0; j< tmpCcnter.length; j++)
						System.out.println(tmpCcnter[j]);
				}
			}
			this.colorCounter = tmpCcnter;
		}
		return this.colorCounter;
	}
	
	/**
	 * Computes the answer (black, white) for the given expected pattern.
	 * 
	 * @param expectedPattern 
	 * @return
	 */
	public Answer computeAnswer(Guess expectedPattern) {
		// First compute black.
		int black = 0;
		for (int i=0; i<colors.length; i++) 
			if (this.colors[i] == expectedPattern.colors[i])
				black++;
		
		int white = 0;
		for (int i=0; i<getColorCounter().length; i++)
			white += Math.min(this.getColorCounter()[i], expectedPattern.getColorCounter()[i]);
		white -= black;
		
		return new Answer(black, white, colors.length);
	}
	
}
